package tournament;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.geom.GeneralPath;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.awt.geom.QuadCurve2D;
import java.util.Date;

/**
 * Class Finalgame.
 * Repraesentiert das Finalspiel des Turniers.
 */
public class FinalGame extends Game {
	
	protected double[] myPointsNow;
	protected double[] myPointsEnd;
	
	protected double[] myPropsNow;
	protected double[] myPropsEnd;

	public FinalGame(int scoreA, int scoreB, Date date) {
		super(0, date);
		
		myPointsNow = new double[8];	// 4 Points
		myPointsEnd = new double[8];	// 4 Points
		
		myPropsNow = new double[4];	// color high/low, line-width, line-style
		myPropsEnd = new double[4];	// color high/low, line-width, line-style
		
		myScoreA = scoreA;
		myScoreB = scoreB;
	}
	
	/**
	 * Initialisierung des Finaspiel-Objekts
	 */	
	public FinalGame(Date date) {
		super(0, date);
		
		myPointsNow = new double[8];	// 4 Points
		myPointsEnd = new double[8];	// 4 Points
		
		myPropsNow = new double[4];	// color high/low, line-width, line-style
		myPropsEnd = new double[4];	// color high/low, line-width, line-style
	}
	
	public Point getTiePoint() {
		return null;
	}
	
	public void setPointsUndecided() {
		setPointsDecided();
	}
	
	public void setPointsDecided() {	

		double x, y, x2, y2;
        
        Point pointA = myNodeA.getTiePoint();
        Point pointB = myNodeB.getTiePoint();
        
        x = pointA.x;
        y = pointA.y;
        
        myPointsEnd[0] = x;
        myPointsEnd[1] = y;
        
        x2 = x + (pointB.x - x) / 2;
        y2 = y;
        
        myPointsEnd[2] = x2;
        myPointsEnd[3] = y2;
                
        x = pointB.x;
        y = pointB.y;
        
        myPointsEnd[4] = x;
        myPointsEnd[5] = y;
        
        x2 = x - (x - pointA.x) / 2;
        y2 = y;
        
        myPointsEnd[6] = x2;
        myPointsEnd[7] = y2;
	}
	
	public void paint(Graphics2D g) {
		myPointsNow = myInterpolator1.getValues();
		myPropsNow = myInterpolator2.getValues();
		
		if (myNodeA.getState() != UNDECIDED && myNodeB.getState() != UNDECIDED) {
			Rectangle r = getRectangle();
			allGames.put(r, this);
			g.setColor(Color.getHSBColor(0f, 0f, 0.12f));
//			g.fillRect(r.x, r.y, r.width, r.height);
		}
		
		paintGame(g);

		myNodeA.paint(g);
		myNodeB.paint(g);
	}
	
	public void paintGame(Graphics2D g) {
		g.setStroke(
				new BasicStroke(
				(float)myPropsNow[2],		// line-width
				BasicStroke.CAP_SQUARE,
				BasicStroke.JOIN_MITER,
				1.0f,
				new float[] { (float)myPropsNow[3], (float)(10.0f - myPropsNow[3]) },
				5.0f));
		
		GeneralPath path;
				
        path = new GeneralPath();
        g.setColor(Color.getHSBColor(0.0f, 0.0f, (float)myPropsNow[1]));
        path.append(new Line2D.Double(myPointsNow[0], myPointsNow[1], myPointsNow[2], myPointsNow[3]), false);      
        g.draw(path);
        
        path = new GeneralPath();
        g.setColor(Color.getHSBColor(0.0f, 0.0f, (float)Math.min(myPropsNow[0], myPropsNow[1])));
        path.append(new Line2D.Double(myPointsNow[2], myPointsNow[3], myPointsNow[6], myPointsNow[7]), false);
        g.draw(path);
        
        path = new GeneralPath();
        g.setColor(Color.getHSBColor(0.0f, 0.0f, (float)myPropsNow[0]));
        path.append(new Line2D.Double(myPointsNow[4], myPointsNow[5], myPointsNow[6], myPointsNow[7]), false);
        g.draw(path);
	}

	public Rectangle getRectangle() {
		Point pointA = myNodeA.getTiePoint();
		Point pointB = myNodeB.getTiePoint();

		return new Rectangle((int)pointA.x, Math.min((int)pointA.y, (int)pointB.y - 50),
							(int)(pointB.x - pointA.x), (int)Math.abs(pointA.y - pointB.y) + 100);
	}
	
	public boolean losesInFuture(Node n) {	
		if( getState() == UNDECIDED )
			return false;
		
		if( ( ( n.getName().compareTo(myNodeA.getName()) == 0 ) && getState() == A ) ||
			( ( n.getName().compareTo(myNodeB.getName()) == 0 ) && getState() == B ) )
			return false;
		
		return true;
	}
	
	protected void setPropsUndecided() {
		myPropsEnd[0] = 1.0;	// color low
		myPropsEnd[1] = 1.0;	// color high
		myPropsEnd[2] = 1.5;	// line-width
		myPropsEnd[3] = 5.0;	// line-style
	}
	
	protected void setPropsDecided() {
		if( myScoreA > myScoreB ) {
			myPropsEnd[0] = 0.5;	// color low
			myPropsEnd[1] = 1.0;	// color high
		}
		else {
			myPropsEnd[0] = 1.0;	// color low
			myPropsEnd[1] = 0.5;	// color high			
		}
		myPropsEnd[2] = 3.0;	// line-width
		myPropsEnd[3] = 10.0;	// line-style
	}
	
	public void update() {
		
		this.setPointsDecided();
		
		switch (getState()) {
		case A:
		case B:
			this.setPropsDecided();
			break;
		case UNDECIDED:
		default:
			this.setPropsUndecided();
			break;

		}
		
		myInterpolator1 = new Interpolator(myPointsNow.clone(), myPointsEnd);
		myInterpolator2 = new Interpolator(myPropsNow.clone(), myPropsEnd);
		
		myNodeA.update();
		myNodeB.update();
	}

	public void init() {
		this.setPointsDecided();
		
		switch (getState()) {
		case A:
		case B:
			this.setPropsDecided();
			break;
		case UNDECIDED:
		default:
			this.setPropsUndecided();
			break;

		}
		
		myPointsNow = myPointsEnd.clone();
		myPropsNow = myPropsEnd.clone();
		
		myInterpolator1 = new Interpolator(myPointsNow.clone(), myPointsEnd);
		myInterpolator2 = new Interpolator(myPropsNow.clone(), myPropsEnd);
		
		myNodeA.init();
		myNodeB.init();
	}
}