package tournament;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.GridLayout;
import java.awt.MouseInfo;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.GeneralPath;
import java.awt.geom.Line2D;
import java.awt.geom.QuadCurve2D;
import java.util.Calendar;
import java.util.Date;
import java.util.Hashtable;
import java.util.Vector;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JTextField;

import tournament.xml.GameResult;

/**
 * Game Class.
 * Repraesentiert ein Spiel eines Turniers.
 */
public class Game extends Node {
	public Node myNodeA;
	public Node myNodeB;

	protected int myScoreA;
	protected int myScoreB;

	public static Hashtable<Rectangle, Game> allGames = new Hashtable<Rectangle, Game>();

	public final static int A = 1;
	public final static int B = 2;
	public final static int UNDECIDED = 0;
	
	protected double[] myPointsNow;
	protected double[] myPointsEnd;
	
	protected double[] myPropsNow;
	protected double[] myPropsEnd;
	
	private GameResult myGameResult;

	public static Vector<Date> dates = new Vector<Date>();
	public static Date sliderDate;
	public Date myDate;

	/**
	 * Initialisierung des Game-Objekts
	 * @param dir Ausrichtung in Bezug auf Turnierbaum
	 * @param gameId ID des Spiels
	 */	
	public Game (int dir, int scoreA, int scoreB, Date date)
	{
		super(dir);
		myScoreA = scoreA;
		myScoreB = scoreB;
		
		System.out.println(scoreA + ":" + scoreB);
		
		construct(date);
	}
	/**
	 * Initialisierung des Game-Objekts
	 * @param dir Ausrichtung in Bezug auf Turnierbaum
	 */	
	public Game(int dir, Date date) {
		super(dir);
		myScoreA = 0;
		myScoreB = 0;

		construct(date);
	}

	private void construct(Date date) {
		
		myDate = date;
		dates.add(date);
				
		myPointsNow = new double[20];	// 10 Points
		myPointsEnd = new double[20];	// 10 Points
		
		myPropsNow = new double[4];	// color high/low, line-width, line-style
		myPropsEnd = new double[4];	// color high/low, line-width, line-style
		
		myInterpolator1 = new Interpolator(myPointsNow.clone(), myPointsEnd);
		myInterpolator2 = new Interpolator(myPropsNow.clone(), myPropsEnd);
	}
	
	/**
	 * Setzen des Spielstatus
	 * @param state Sieger A/B oder UNDECIDED
	 */	
	public void setState(int scoreA, int scoreB) {
		myScoreA = scoreA;
		myScoreB = scoreB;
	}

	public Point getTiePoint() {
		int y;

		switch (getState()) {
		case A:
			y = myNodeA.getTiePoint().y;
			break;
		case B:
			y = myNodeB.getTiePoint().y;
			break;
		case UNDECIDED:
		default:
			y = (myNodeA.getTiePoint().y + myNodeB.getTiePoint().y) / 2;
			break;

		}

		return new Point(myNodeA.getTiePoint().x + (DIR * Tournament.STEP), y);
	}

	/**
	 * Zeichnen des Spiels
	 */	
	public void paint(Graphics2D g) {
		myPointsNow = myInterpolator1.getValues();
		myPropsNow = myInterpolator2.getValues();
		
		paintGame(g);

		myNodeA.paint(g);
		myNodeB.paint(g);
	}

	/**
	 * Paints the rectangle, which describes a game.  
	 * @param x x-value of the mousePointer
	 * @param y y-value of the mousePointer
	 */
	public void paintRectangle(Graphics2D g, int x, int y)
	{
		if (myNodeA.getState() != UNDECIDED && myNodeB.getState() != UNDECIDED) 
		{
			Rectangle r = getRectangle();
			allGames.put(r, this);
			if (r.contains(x, y))
			{
				g.setColor(Color.getHSBColor(0f, 0f, 0.12f));
				g.fillRect(r.x, r.y, r.width, r.height);
			}
		}
		myNodeA.paintRectangle(g, x, y);
		myNodeB.paintRectangle(g, x, y);
	}
	
	/**
	 * Liefert Namen des Gewinnerteams oder leeren Namen bei Unentschieden
	 * @return Name
	 */	
	public String getName() {
		switch (getState()) {
		case A:
			return myNodeA.getName();
		case B:
			return myNodeB.getName();
		case UNDECIDED:
		default:
			return "";
		}
	}
	/**
	 * Liefert Namen des Verliererteams oder leeren Namen bei Unentschieden
	 * @return Name
	 */	
	public String getNameInversed() {
		switch (getState()) {
		case B:
			return myNodeA.getName();
		case A:
			return myNodeB.getName();
		case UNDECIDED:
		default:
			return "";
		}
	}
	
	/**
	 * Setzt Koordinaten fuer Animation bei Unentschieden
	 */	
	protected void setPointsUndecided() {
		double x, y, x2, y2, x3, y3;

		Point pointA = myNodeA.getTiePoint();
		Point pointB = myNodeB.getTiePoint();

		x = pointA.x;
		y = pointA.y;

		myPointsEnd[0] = pointA.x;
		myPointsEnd[1] = pointA.y;
		
		x2 = x + (DIR * Tournament.STEP1);
		y2 = y;
		
		myPointsEnd[2] = x2;
		myPointsEnd[3] = y2;

		x = x2;
		y = y2;

		x2 = x + (DIR * Tournament.STEP2);
		y2 = y;

		myPointsEnd[4] = x2;
		myPointsEnd[5] = y2;
		
		x3 = x2;
		y3 = y2 + Tournament.STEPY;
		
		myPointsEnd[6] = x3;
		myPointsEnd[7] = y3;

		x = x3;
		y = y3;

		x2 = x;
		y2 = pointB.y - Tournament.STEPY;

		myPointsEnd[8] = x2;
		myPointsEnd[9] = y2;

		x = x2;
		y = y2;

		x2 = x;
		y2 = pointB.y;

		myPointsEnd[10] = x2;
		myPointsEnd[11] = y2;
		
		x3 = x - (DIR * Tournament.STEP2);
		y3 = pointB.y;

		myPointsEnd[12] = x3;
		myPointsEnd[13] = y3;
		
		x = x3;
		y = y3;

		x2 = pointB.x;
		y2 = pointB.y;


		myPointsEnd[14] = x2;
		myPointsEnd[15] = y2;
		
		x = pointA.x + (DIR * Tournament.STEP1) + (DIR * Tournament.STEP2);
		y = (pointA.y + pointB.y) / 2;
		
		myPointsEnd[16] = x;
		myPointsEnd[17] = y;
		
		x2 = x + (DIR * Tournament.STEP3);
		y2 = y;
		
		myPointsEnd[18] = x2;
		myPointsEnd[19] = y2;
	}

	/**
	  * Setzt Linieneigenschaften fuer Animation bei Unentschieden
	  */		
	protected void setPropsUndecided() {
		myPropsEnd[0] = 1.0;	// color low
		myPropsEnd[1] = 1.0;	// color high
		myPropsEnd[2] = 1.5;	// line-width
		myPropsEnd[3] = 5.0;	// line-style
	}

	/**
	 * Setzt Linieneigenschaften fuer Animation bei feststehende Spielentscheidung
	 */		
	protected void setPropsDecided() {
		myPropsEnd[0] = 0.5;	// color low
		if( myParent.losesInFuture(this) == false )
			myPropsEnd[1] = 1.0;	// color high
		else
			myPropsEnd[1] = 0.5;	// color high
		myPropsEnd[2] = 3.0;	// line-width
		myPropsEnd[3] = 10.0;	// line-style
	}
	
	/**
	 * Setzt Koordinaten fuer Animation bei feststehende Spielentscheidung
	 */	
	protected void setPointsDecided() {

		double x, y, x2, y2, x3, y3;

		Point pointA = myNodeA.getTiePoint();
		Point pointB = myNodeB.getTiePoint();

		x = pointA.x;
		y = pointA.y;
		
		myPointsEnd[0] = x;
		myPointsEnd[1] = y;

		x2 = x + (DIR * Tournament.STEP1) + (DIR * Tournament.STEP2);
		y2 = y;

		myPointsEnd[2] = x2;
		myPointsEnd[3] = y2;
		
		x = x2;
		y = y2;

		x2 = x + 0;
		y2 = y;
		
		myPointsEnd[4] = x2;
		myPointsEnd[5] = y2;

		x3 = x2;
		y3 = y2 + 0;
		
		myPointsEnd[6] = x3;
		myPointsEnd[7] = y3;

		x = x3;
		y = y3;

		x2 = x;
		y2 = pointB.y - 0;

		myPointsEnd[8] = x2;
		myPointsEnd[9] = y2;

		x = x2;
		y = y2;

		x2 = x;
		y2 = pointB.y;
		
		myPointsEnd[10] = x2;
		myPointsEnd[11] = y2;

		x3 = x - 0;
		y3 = pointB.y;
		
		myPointsEnd[12] = x3;
		myPointsEnd[13] = y3;

		x = x3;
		y = y3;

		x2 = pointB.x;
		y2 = pointB.y;

		myPointsEnd[14] = x2;
		myPointsEnd[15] = y2;

		if (getState() == A)
			y = pointA.y;
		else
			y = pointB.y;

		x = pointA.x;

		myPointsEnd[16] = x;
		myPointsEnd[17] = y;
		
		x2 = x + (DIR * Tournament.STEP);
		y2 = y;
		
		myPointsEnd[18] = x2;
		myPointsEnd[19] = y2;
	}
	
	/**
	 * Zeichen des Spiels
	 */	
	protected void paintGame(Graphics2D g) {

		g.setStroke(
				new BasicStroke(
				(float)myPropsNow[2],		// line-width
				BasicStroke.CAP_SQUARE,
				BasicStroke.JOIN_MITER,
				1.0f,
				new float[] { (float)myPropsNow[3], (float)(10.0f - myPropsNow[3]) },
				5.0f));
		
		g.setColor(Color.getHSBColor(0.0f, 0.0f, (float)myPropsNow[0]));

		GeneralPath path = new GeneralPath();
		
		path.append(new Line2D.Double(myPointsNow[0], myPointsNow[1], myPointsNow[2], myPointsNow[3]), true);
		path.append(new QuadCurve2D.Double(myPointsNow[2], myPointsNow[3],
				myPointsNow[4], myPointsNow[5], myPointsNow[6], myPointsNow[7]), true);
		
		path.append(new Line2D.Double(myPointsNow[6], myPointsNow[7], myPointsNow[8], myPointsNow[9]), true);

		path.append(new QuadCurve2D.Double(myPointsNow[8], myPointsNow[9],
				myPointsNow[10], myPointsNow[11], myPointsNow[12], myPointsNow[13]), true);

		path.append(new Line2D.Double(myPointsNow[12], myPointsNow[13], myPointsNow[14], myPointsNow[15]), true);

		g.draw(path);

		path = new GeneralPath();

		g.setColor(Color.getHSBColor(0.0f, 0.0f, (float)myPropsNow[1]));

		path.append(new Line2D.Double(myPointsNow[16], myPointsNow[17], myPointsNow[18], myPointsNow[19]), false);

		g.draw(path);
	}

	/**
	 * Liefert den Rechteckbereich fuer den per Klick ein Editierdialog geoeffnet wird
	 * @return Rechteckbereich
	 */	
	public Rectangle getRectangle() {
		Point pointA = myNodeA.getTiePoint();
		Point pointB = myNodeB.getTiePoint();

		int height = (int) pointB.y - (int) pointA.y;
		int width = Tournament.STEP;
		
		//System.out.println(height + " " + width);
		
		if ((DIR * Tournament.STEP) < 0) {
			return new Rectangle((int) pointA.x - width * 2/3, (int) pointA.y + height / 3,
					width / 3, height / 3);
		} else {
			return new Rectangle((int) pointA.x + width / 3, (int) pointA.y + height / 3,
					width / 3, height / 3 );
		}
	}

	/**
	 * Oeffnen des Editierdialogs zur Anpassung des Spielergebnis
	 */	
	public void dialogBox() {
		
		final JDialog dialog = new JDialog((JFrame)null, "Spielergebnis", true);

		JTextField resultA = new JTextField(Integer.toString(myScoreA), 10);
		JTextField resultB = new JTextField(Integer.toString(myScoreB), 10);
		JButton closeButton = new JButton("Ok");
		
		
				
		ActionListener myAction = new ActionListener() {

			public void actionPerformed(ActionEvent e) {
//				JButton button = (JButton)e.getSource();
//				JDialog dialog = (JDialog)button.getParent().getParent().getParent().getParent();
				
				dialog.setVisible(false);
			}
		};
		closeButton.addActionListener( myAction );
		resultA.addActionListener( myAction );
		resultB.addActionListener( myAction );
		
		dialog.setLayout(new GridLayout(5, 1));
		
		dialog.add(new JLabel(myNodeA.getName()));
		dialog.add(resultA);
		dialog.add(new JLabel(myNodeB.getName()));
		dialog.add(resultB);
		dialog.add(closeButton);
		dialog.setResizable(false);
		dialog.pack();
		Point p = MouseInfo.getPointerInfo().getLocation();
		dialog.setLocation(p.x -50, p.y-50);
		
		// Modal dialog box
		dialog.setVisible( true );
		
		try
		{
			int scoreA = Integer.parseInt(resultA.getText());
			int scoreB = Integer.parseInt(resultB.getText());

			myScoreA = new Integer(scoreA);
			myScoreB = new Integer(scoreB);
		} catch (NumberFormatException e1)
		{
			System.out.println(e1.getMessage());
		}
	}

	/**
	 * Liefert den Status des Spiels
	 * A/B - Sieger Mannschaft oder noch unentschieden (UNDECIDED)
	 */
	public int getState() {
		
		if( sliderDate.before(myDate) || undecidedChild() )
			return UNDECIDED;
		else
			if( myScoreA == myScoreB )
				return Game.UNDECIDED;
			else if ( myScoreA > myScoreB )
				return Game.A;
			else
				return Game.B;
	}
	

	/**
	 * Stellt fest ob die Mannschaft in einer hoeheren Turnierebene ein Spiel verloren hat
	 * @param n Node-Objekt fuer das die Ueberpruefung durchgefuehrt wird
	 * @return true wenn ein Spiel verloren wurde
	 */	
	public boolean losesInFuture(Node n) {	
		if( getState() == UNDECIDED )
			return false;
		
		if( ( ( n.getName().compareTo(myNodeA.getName()) == 0 ) && getState() == A ) ||
			( ( n.getName().compareTo(myNodeB.getName()) == 0 ) && getState() == B ) )
			return myParent.losesInFuture(n);
		
		return true;
	}
	
	
	/**
	 * Setzen des zugehoerigen Nodes in der naechsthoeheren Ebene
	 * @param n Elternknoten
	 */
	public void setParentNode(Node n) {
		myParent = n;
		
		myNodeA.setParentNode(this);
		myNodeB.setParentNode(this);
	}

	
	/**
	 * Stellt fest ob auf einer niedrigeren Ebene ein Vorgaengerspiel den Status unentschieden hat
	 * @return true wenn ein Spiel noch unentschieden ist
	 */			
	public boolean undecidedChild() {
		
		if( myScoreA == myScoreB )
			return true;
		
		return myNodeA.undecidedChild() || myNodeB.undecidedChild();
	}
	
	
	/**
	 * Neuberechnung der Zielwerte der Animation
	 */	
	public void update() {
		switch (getState()) {
		case A:
		case B:
			this.setPropsDecided();
			this.setPointsDecided();
			break;
		case UNDECIDED:
		default:
			this.setPropsUndecided();
			this.setPointsUndecided();
			break;

		}
		
		myInterpolator1 = new Interpolator(myPointsNow.clone(), myPointsEnd);
		myInterpolator2 = new Interpolator(myPropsNow.clone(), myPropsEnd);
		
		myNodeA.update();
		myNodeB.update();
	}
	
	
	/**
	 * Initialisierung der Ziel- und Startwerte der Animation
	 */	
	@Override
	public void init() {
		switch (getState()) {
		case A:
		case B:
			this.setPropsDecided();
			this.setPointsDecided();
			break;
		case UNDECIDED:
		default:
			this.setPropsUndecided();
			this.setPointsUndecided();
			break;

		}
		
		myPointsNow = myPointsEnd.clone();
		myPropsNow = myPropsEnd.clone();
		
		myInterpolator1 = new Interpolator(myPointsNow.clone(), myPointsEnd);
		myInterpolator2 = new Interpolator(myPropsNow.clone(), myPropsEnd);
		
		myNodeA.init();
		myNodeB.init();
	}
}
