package tournament;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.font.FontRenderContext;
import java.awt.font.TextLayout;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Vector;

import javax.swing.ImageIcon;

/**
 * Player Class.
 * Stellt einen Teilnehmer eines Turniers dar.
 */
public class Player extends Node {
	public String myName;
	public Point myTiePoint;
	public Interpolator myInterpolator;
	
	public static HashMap<Rectangle2D, Player> allPlayers = new HashMap<Rectangle2D, Player>();
	public static Vector<Player> allPlayersVec = new Vector<Player>();
	
	protected double[] myPropsNow;
	protected double[] myPropsEnd;
	
	protected boolean drawHover = false;
	private ImageIcon myIcon;

/**
 * Intialisierung eines Player-Objekts
 * @param name	Name der Mannschaft bzw. des Spielers
 * @param dir	Ausrichtung im Bezug auf den Turnierbaum (links- oder rechtsseitig, 1 bzw. -1)
 */
	public Player(String name, int dir, ImageIcon icon) {
		super(dir);
		myName = name;
		
		myPropsNow = new double[1];	// color
		myPropsEnd = new double[1];	// color
		myInterpolator = new Interpolator(myPropsNow.clone(), myPropsEnd);
		myIcon = icon;
		
		allPlayersVec.add(this);

	}
	
/**
 * Liefert den Namen der Mannschaft bzw. des Spielers
 */
	public String getName() {
		return myName;
	}

/**
 * Liefert den Anschlusspunkt zum Zeichnen des Baumes
 */
	public Point getTiePoint() {

		return myTiePoint;
	}

/**
 * Zeichnen des Player-Objekts.
 * Darstellung von Name und eines eventuell vorhandenen Symbols
 */	
	public void paint(Graphics2D g) {
		myPropsNow = myInterpolator.getValues();
		
        FontRenderContext frc = g.getFontRenderContext();
        TextLayout tl = new TextLayout(myName, g.getFont(), frc);
               
        g.setColor(Color.getHSBColor(0.0f, 0.0f, (float)myPropsNow[0]));
   
        float x, y;
        
        x = myTiePoint.x;
        
        if( losesInFuture(this) == false && myIcon != null ) {
        	
        	if( DIR < 0 )
        		x = x + 10;
        	else
        		x = x - (myIcon.getIconWidth() + 10);
        	
        	g.drawImage(myIcon.getImage(), (int) x,
        			myTiePoint.y - myIcon.getIconHeight() / 2, null);
        	
        	if( DIR < 0 )
        		x = x + myIcon.getIconWidth();
        }
        
        if( DIR > 0 )
        {
        	x = x - tl.getAdvance() - 10;
        	y = (float)myTiePoint.y + tl.getAscent() * 0.4f;
        } else 
        {
        	x = x + 10;
        	y = (float)myTiePoint.y + tl.getAscent() * 0.4f;
        }
        	
        tl.draw(g, x, y);
        
        

        Rectangle2D rect = tl.getBounds();
        int borderWidth = 5;
        rect.setRect(x-borderWidth, y-rect.getHeight()-borderWidth, rect.getWidth()+borderWidth*2, rect.getHeight()+borderWidth*2);
        
        allPlayers.put(rect, this);
        
        if (drawHover)
		{
			g.setStroke(new BasicStroke());
		//	g.draw(rect);
		}
    }

/**
 * Liefert den Status des Spiels
 * A/B - Sieger Mannschaft oder noch unentschieden (UNDECIDED)
 */
	public int getState() {
		return Game.A;
	}
/**
 * Setzen des zugehoerigen Nodes in der naechsthoeheren Ebene
 * @param n Elternknoten
 */
	public void setParentNode(Node n) {
		myParent = n;		
	}

/**
 * Stellt fest ob die Mannschaft in einer hoeheren Turnierebene ein Spiel verloren hat
 * @param n Node-Objekt fuer das die Ueberpruefung durchgefuehrt wird
 * @return true wenn ein Spiel verloren wurde
 */	
	public boolean losesInFuture(Node n) {
		return myParent.losesInFuture(this);
	}

/**
 * Stellt fest ob auf einer niedrigeren Ebene ein Vorgaengerspiel den Status unentschieden hat
 * @return true wenn ein Spiel noch unentschieden ist
 */	
	public boolean undecidedChild() {
		return false;
	}

/**
 * Neuberechnung der Zielwerte der Animation
 */	
	public void update() {
		if( myParent.losesInFuture(this) == true )
        	myPropsEnd[0] = 0.5;
        else
        	myPropsEnd[0] = 1.0;
		
		myInterpolator = new Interpolator(myPropsNow.clone(), myPropsEnd);
	}

/**
 * Initialisierung der Ziel- und Startwerte der Animation
 */	
	public void init() {
		update();
		
		myPropsNow = myPropsEnd.clone();
		
		myInterpolator = new Interpolator(myPropsNow.clone(), myPropsEnd);
	}

	@Override
	public void paintRectangle(Graphics2D g, int x, int y)
	{
		//noop		
	}
	
	public static List<String> getAllNames()
	{
		ArrayList<String> retList = new ArrayList<String>();
		for (Player player:allPlayersVec)
		{
			retList.add(player.getName());
		}
		return retList;
	}
}
