package tournament;

import java.awt.BasicStroke;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.geom.Line2D;
import java.awt.Color;

public class Tipp {
	
	private float thickness = 20.0f;
	private float length = 100.0f;
	private float x = 20.0f;
	private float y = 20.0f;
	
	private int maxState = 2;
	private int tippState = 0;
	private int realState = 0;
	
	private boolean drawClipping = false;
	private boolean teamEliminated = false;
	
	private Rectangle bbox = new Rectangle();
	
	public Tipp (float x, float y, float length, float thickness, int maxState)
	{
		this.x = x;
		this.y = y;
		this.length = length;
		this.thickness = thickness;
		this.maxState = maxState;
	}
	
	public void setTippDimensions (float x, float y, float length, float thickness)
	{
		this.x = x;
		this.y = y;
		this.length = length;
		this.thickness = thickness;
	}
	
	public void paint(Graphics2D g2)
	{
		LineStyle[] lineStyles = new LineStyle[tippState];
			
		for (int i=0; i < lineStyles.length; i++)
		{
			if (i < realState)
				lineStyles[i] = LineStyle.GREEN;
			else 
			{
				if (teamEliminated)
					lineStyles[i] = LineStyle.RED;
				else
				lineStyles[i] = LineStyle.UNDECIDED;
			}
		}
	
		paintTipp(g2, lineStyles);		 
	}
	
	public enum LineStyle
	{
		GREEN( new Color(32,142,132) ), //Color.GREEN),
		RED( new Color(236,28,35) ), //Color.RED),
		UNDECIDED( new Color(32,142,132) ); //Color.GREEN),
		
		private LineStyle (Color col)
		{
			this.color = col;
		}
		
		public Color color;
	}
	
	private void drawCap(Graphics2D g2, Color color, Rectangle clipRect, float x)
	{
		if (drawClipping)
		{
			g2.setColor(Color.WHITE);
			g2.setStroke(new BasicStroke());
			g2.draw(clipRect);
		}
		
		g2.setClip(clipRect);
		
		BasicStroke bs = new BasicStroke(thickness, BasicStroke.CAP_ROUND, BasicStroke.JOIN_MITER); 
		g2.setStroke(bs); 
		Line2D line = new Line2D.Float(x, y, (x+1.0f), y); 
		
		g2.setColor(color);
		g2.draw(line); 
		
		g2.setClip(null);
	}
	
	
	public void paintTipp(Graphics2D g2, LineStyle ... lineStyles)
	{
		float dash[] = {thickness/3, thickness/3};
		float mx = x;
		
		Line2D line = new Line2D.Float();
		Rectangle rect = new Rectangle();		
		BasicStroke bs1 = new BasicStroke(thickness, BasicStroke.CAP_BUTT, BasicStroke.JOIN_MITER); 
		BasicStroke bs2 = new BasicStroke(thickness, BasicStroke.CAP_BUTT, BasicStroke.JOIN_MITER, 1, dash, thickness/3); 

		Color col = LineStyle.GREEN.color;
		
		if (lineStyles.length > 0 && (lineStyles[0] == LineStyle.RED))
			col = lineStyles[0].color;
		

		rect.setBounds((int)(x-thickness/2), (int)(y-thickness/2)-1, (int)(thickness/2)+2, (int)(thickness)+2);
		if (length < 0)
		{
			rect.x = (int)x; 
		}
		bbox.setBounds(rect);
		
		drawCap(g2, col, rect, mx);
	
		
		for (int i = 0; i < lineStyles.length; i++)
		{
			g2.setColor(col = lineStyles[i].color);
			
			if (lineStyles[i]==LineStyle.UNDECIDED)
			{
				g2.setStroke(bs2);
				float diff = lineStyles.length-i;
				if (lineStyles.length == maxState   && lineStyles[lineStyles.length-2]==LineStyle.UNDECIDED) diff -= 1;		
				else if (lineStyles.length == maxState   && lineStyles[lineStyles.length-1]==LineStyle.UNDECIDED) diff -= 0.5f;
				if (lineStyles.length == maxState-1 && lineStyles[lineStyles.length-1]==LineStyle.UNDECIDED) diff -= 0.5f;
//				System.out.println(diff);
				line.setLine(mx, y, mx+length*(diff), y);
				mx+=length*diff;
				g2.draw(line);
				break;
			}
			
//			g2.setStroke(bs1);
//			float len = mx+length;
//			if (maxState-i <= 2) len -= length*0.5f;
//
//			line.setLine(mx, y, len, y);
//			if (length< 0)
//				mx=len+1;
//			else
//				mx=len-1;
			
			g2.setStroke(bs1);
			float myLen = length;
			
			if (i == maxState-2 || i == maxState -1) 
				myLen= length*0.5f;
			
			line.setLine(mx, y, mx+myLen, y);
			if (length< 0)
				mx+=myLen+1;
			else
				mx+=myLen-1;
			
			g2.draw(line);
		}
		mx++;
		
		
		rect.setBounds((int)(mx), (int)(y-thickness/2)-1, (int)(thickness/2)+2, (int)(thickness)+2);
		if (length < 0)
		{
			rect.x = (int)(mx-thickness/2); 
		}
		bbox.add(rect);
		drawCap(g2, col, rect, mx);
		
		if (drawClipping)
		{
			g2.setColor(Color.WHITE);
			g2.setStroke(new BasicStroke());
			g2.draw(bbox);
		}
	}

	public boolean containsPoint(int x, int y)
	{
		return bbox.contains(x, y);
	}
	
	/**
	 * raises the tipp by one game.
	 * @return the current state
	 */
	public int moveTippStateUp()
	{
		if (tippState < maxState)
			tippState++;
		
		return tippState;
	}
	
	/**
	 * lowers the tipp by one game
	 * @return the currrent state
	 */
	public int moveTippStateDown()
	{
		if (tippState > 0)
			tippState --;
		
		return tippState;
	}
	
	/**
	 * gets the current state
	 * @return the currrent state
	 */
	public int getTippState()
	{
		return tippState;
	}
	
	/**
	 * sets the tippState
	 * @param tippState the tippstate
	 */
	public void setTippState(int tippState)
	{
		this.tippState = tippState;
	}

	/**
	 * set to true if this team got eliminated.
	 * @param teamEliminated true if team was eliminated<br>false otherwise
	 */
	public void setTeamEliminated(boolean teamEliminated)
	{
		this.teamEliminated = teamEliminated;
	}

	/**
	 * sets the real state (the amount of won games) for this tipp
	 * @param realState the real state
	 */
	public void setRealState(int realState)
	{
		if (realState >= 0 && realState <= maxState)
			this.realState = realState;
	}
	
	/**
	 * move the real state up by one game.
	 */
	public void moveRealStateUp()
	{
		if (realState < maxState)
			realState ++;
	}
	

}
