package tournament;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.MouseInfo;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.geom.Rectangle2D;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import tournament.xml.GameResult;
import tournament.xml.Team;
import tournament.xml.TournamentData;

/**
 * Tournament Class.
 * Repraesentiert den Turnierbaum als auch die zugehoerige Zeichenflaeche (JPanel).
 */
public class Tournament extends JPanel implements MouseListener, ChangeListener, MouseMotionListener {

	public static int STEP;
	public static int STEP1;
	public static int STEP2;
	public static int STEP3;
	public static int STEPY;
	public static int dimension = 16;
	
	public FinalGame myFinalGame;
	ArrayList<Node> myLeft;
	ArrayList<Node> myRight;
	
	private static DateFormat myDateFormat = new SimpleDateFormat("dd.MM.yy HH:mm");
	
	public TippManager tippManager = null;
	
	public InfoPanel infoPanel = null;
	
	private Cursor handCursor = new Cursor(Cursor.HAND_CURSOR);
	
	public TournamentData tournamentData = new TournamentData();
  
	public JSlider mySlider;
	
	public JPanel slidePanel;
	
	private boolean repaintOnce = false;
	
	/**
	 * Neuzeichnen der Zeichenflaeche (JPanel)
	 * @param g Zeichenflaeche
	 */
    public void paint(Graphics g) {
        Graphics2D g2 = (Graphics2D) g;
        Dimension d = getSize();
        g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                            RenderingHints.VALUE_ANTIALIAS_ON);
        g2.setBackground(Color.black);
        g2.clearRect(0, 0, d.width, d.height);


        Game.allGames.clear();
		Player.allPlayers.clear();
        if (this.isShowing())
        {
	        Point p1 = getLocationOnScreen();
	        Point p2 = MouseInfo.getPointerInfo().getLocation();
	        myFinalGame.paintRectangle(g2, p2.x - p1.x, p2.y - p1.y);
        }
        tippManager.paint(g2);
        myFinalGame.paint(g2);


    }
    
    public void createTipps(int x, int xend, int y, int ystep1, int ystep2, int length)
    {
    	if (tippManager == null)
    	{
    		tippManager = new TippManager(Player.getAllNames());
    		this.addMouseListener(tippManager);
    		this.addMouseMotionListener(tippManager);
    	}
    	tippManager.setSize(x+STEP2+STEP1, xend-STEP2-STEP1, y, ystep1, ystep2, length, 15);
    	tippManager.updateTippStates();
    }
    
    private void resetTipps()
    {
    	this.removeMouseListener(tippManager);
    	this.removeMouseMotionListener(tippManager);
    	tippManager = null;
    }
    
 

	/**
	 * Rekursive Funktion zur Erzeugung des Turnierbaums.
	 * Node-Objekte werden paarweise zu Game-Objekten verknuepft bis ein einzelnes Game-Objekt uebrig bleibt.
	 * @param l	Liste der zu verknuepfenden Node-Objekte
	 * @return	Einzelnes Node-Objekt das nicht mehr verknuepft werden kann.
	 */
    public static Node buildTree(ArrayList<Node> l, int gameId, TournamentData tData) {
    	int s = l.size();
    	
    	if( s == 1 )
    		return (Node)l.get(0);
    	    	
    	if( s % 2 != 0 )
    		return null;
    	    	
    	ArrayList<Node> l2 = new ArrayList<Node>();
    	
    	Iterator<Node> it;
    	
    	it = l.iterator();
    	
    	while( it.hasNext() ) {
    		Node n = it.next();
    		Node n2 = it.next();
    		
    		System.out.println("GameID: " + gameId);
    		GameResult gr = tData.getGame(gameId);

    		Game g = new Game(n.DIR, gr.team1, gr.team2, gr.getDate());    		
    		
    		gameId++;
    		
    		g.myNodeA = n;
    		g.myNodeB = n2;

    		l2.add(g);
    	}
    	
    	return buildTree(l2, gameId, tData);
    }
    
	/**
	 * Abarbeitung von Mausklicks. Aufruf des Dialogfensters zur Bearbeitung von Spielausgaengen.
	 */
    public void buildTournamentTree(int teamCount, List<String> players)
	{
//    	Tournament.players = players;
    	Player.allPlayers.clear();
    	Player.allPlayersVec.clear();
    	Game.allGames.clear();
    	Game.dates.clear(); 	
    	resetTipps();
    	
		ArrayList<Node> l1 = new ArrayList<Node>();
		ArrayList<Node> l2 = new ArrayList<Node>();
		
		int teamHalf = Math.round(teamCount / 2);
		int j = 0;

		for (int i = 0; i < teamHalf; i++)
		{
			Team t = this.tournamentData.getTeamByName(players.get(j));
			ImageIcon icon = null;
			
			if( t != null )
			{
				try {
					icon = new ImageIcon(t.getIcon());
				}
				catch(Exception e) {
					icon = null;
				}				
			}				
			
			l1.add(new Player(players.get(j++), 1, icon));
		}
		
		int gameId = 1;
		
		Node n1 = buildTree(l1, gameId, this.tournamentData);

		gameId = l1.size();
		
		for (int i = 0; i < teamHalf; i++)
		{
			Team t = this.tournamentData.getTeamByName(players.get(j));
			ImageIcon icon = null;
			
			if( t != null )
			{
				try {
					icon = new ImageIcon(t.getIcon());
				}
				catch(Exception e) {
					icon = null;
				}				
			}				
			l2.add(new Player(players.get(j++), -1, icon));
		}

		Node n2 = buildTree(l2, gameId, this.tournamentData);

		GameResult gr = this.tournamentData.getGame( l1.size() * 2 - 1 );
		
		myFinalGame = new FinalGame(gr.team1, gr.team2, gr.getDate());
		
		myFinalGame.myNodeA = n1;
		myFinalGame.myNodeB = n2;
		myFinalGame.setParentNode(null);

		myLeft = l1;
		myRight = l2;

		Collections.sort(Game.dates);
		Calendar cal = Calendar.getInstance();
		cal.setTime(Game.dates.get(0));
		cal.add(Calendar.DAY_OF_MONTH, -1);
		Game.dates.insertElementAt(cal.getTime(), 0);
		
		int mid = Game.dates.size() / 2;
		int end = Game.dates.size() - 1;
		
		Hashtable<Integer, JLabel> labelTable = new Hashtable<Integer, JLabel>();
		
		labelTable.put( new Integer( 0 ), new JLabel(myDateFormat.format(Game.dates.get(0))));
		labelTable.put( new Integer( mid ), new JLabel(myDateFormat.format(Game.dates.get(mid))));
		labelTable.put( new Integer( end ), new JLabel(myDateFormat.format(Game.dates.get(end))));
		
		slidePanel.remove(mySlider);
		mySlider = new JSlider();
		slidePanel.add(mySlider, BorderLayout.PAGE_END);

		mySlider.setBorder(BorderFactory.createTitledBorder("Date"));   
		mySlider.setLabelTable(labelTable);
        mySlider.setPaintLabels(true);
		mySlider.setPaintTicks(true);
		mySlider.setSnapToTicks(true);
		mySlider.setMajorTickSpacing(1);
        
        mySlider.setMinimum(0);
		mySlider.setMaximum(Game.dates.size() - 1);
		mySlider.setValue(0);
		
		mySlider.addChangeListener(this); 

		Game.sliderDate = Game.dates.get(0);
		
		if (infoPanel!=null){
			infoPanel.setDate(Game.sliderDate);
			infoPanel.clear();
		}
		
		this.rearrange(getSize().width, getSize().height);

		Interpolator.run = false;
		myFinalGame.init();
		Interpolator.reset();
		Interpolator.run = true;
		Interpolator.reset();
		
		
		this.paint(this.getGraphics());
	}
    
	public void mouseClicked(java.awt.event.MouseEvent e) {
		Point point = e.getPoint();
		
		for( Rectangle r : Game.allGames.keySet() ) {
			if( r.contains( point ) ) {
				Game.allGames.get( r ).dialogBox();
				Interpolator.run = false;
				myFinalGame.update();
				Interpolator.reset();
				Interpolator.run = true;
				return;
			}			
		}
		

	}

	@Override
	public void mouseEntered(java.awt.event.MouseEvent e) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void mouseExited(java.awt.event.MouseEvent e) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void mousePressed(java.awt.event.MouseEvent e) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void mouseReleased(java.awt.event.MouseEvent e) {
		// TODO Auto-generated method stub
		
	}
	
	@Override
	public void mouseDragged(MouseEvent e)
	{
		// TODO Auto-generated method stub
	}

	@Override
	public void mouseMoved(MouseEvent e)
	{
		if ((tippManager!= null) &&tippManager.containsCursor(e))
		{
			this.setCursor(handCursor);
			return;
		}
		
		for (Rectangle2D rect:Player.allPlayers.keySet())
		{
			if (rect.contains(e.getX(), e.getY()))
			{
				String name = Player.allPlayers.get(rect).getName();
				Team team = tournamentData.getTeamByName(name);
				if (team!=null)
					infoPanel.setData(team.getIcon(), team.getName(), team.getDescription());
				this.setCursor(handCursor);
				return;
			}
				
		}
		
		for (Game game:Game.allGames.values())
		{
			if (game.getRectangle().contains(e.getX(), e.getY()))
			{
				this.setCursor(handCursor);
				repaint();
				repaintOnce = true;
				return;
			}
		}
		
		this.setCursor(null);
		
		if (repaintOnce)
		{
			repaintOnce = false;
			repaint();
		}
	}

	/**
	 * Anpassung des aktuellen Zeitpunkts des Baumzustandes bei Veraenderung des Zeitschiebers
	 */
	public void stateChanged(ChangeEvent e) {
		JSlider slider = (JSlider)e.getSource();
		
		Game.sliderDate = Game.dates.get(slider.getValue());
		
		if (infoPanel!=null)
			infoPanel.setDate(Game.sliderDate);
		
		Interpolator.run = false;
		myFinalGame.update();
		Interpolator.reset();
		Interpolator.run = true;
	}

	/**
	 * Neuanordnung des Baumes bei Veraenderung der Fenstergroesse.
	 * 
	 * @param width
	 *            Fensterbreite
	 * @param height
	 *            Fensterhoehe
	 */	
	public void rearrange(int width, int height) {
		

		int size = myLeft.size();
		
		final int yStart = 20;
		final int xStart = 150;
		final int step = 20, step1 = 5, step2 = 5;
		
        int y, x;

        int inc1 = 20, inc2 = 10;
        
                
        int minHeight = 2 * yStart + (size / 2) * inc1 + ((size / 2) - 1) * inc2;
        int treeWidth = (int)(Math.log(size) / Math.log(2) * 2 + 1) * step;
        int minWidth = 2 * xStart + treeWidth;
        
        float scaleH = 1.0f;
        float scaleW = 1.0f;
        
        if( minHeight > height )
        	height = minHeight;
        else {
        	int h = height - 2 * yStart;
        	
        	minHeight = minHeight - 2 * yStart;
        	
        	scaleH = (float)h / (float)minHeight; 
        	
        	inc1 = (int)(inc1 * scaleH);
        	inc2 = (int)(inc2 * scaleH);        	
        }
        
        if( minWidth > width )
        	width = minWidth;
        else {
        	int w = width - 2 * xStart;
        	
        	minWidth = minWidth - 2 * xStart;
        	
        	scaleW = (float)w / (float)minWidth;
        }
        
        float scale = Math.min(scaleW, scaleH);
        
        STEP = (int)(step * scaleW);
    	STEP1 = (int)(step1 * scale);
    	STEP2 = (int)(step2 * scale);
    	STEP3 = STEP - STEP1 - STEP2;
    	STEPY = STEP2;
        
        this.setPreferredSize(new Dimension(width, height));
        
        y = yStart;
        x = xStart;
        
        Iterator<Node> it = myLeft.iterator();
        
        while( it.hasNext() ) {
        	Player p = (Player)it.next();
        	p.myTiePoint = new Point(x, y);
        	
        	y = y + inc1;
        	
        	Player p2 = (Player)it.next();
        	p2.myTiePoint = new Point(x, y);
        	
        	y = y + inc2;
        }
        
        y = yStart;
        x = xStart + (int)(Math.log(size) / Math.log(2) * 2 + 1) * STEP;
        
        it = myRight.iterator();
        
        while( it.hasNext() ) {
        	Player p = (Player)it.next();
        	p.myTiePoint = new Point(x, y);
        	
        	y = y + inc1;
        	
        	Player p2 = (Player)it.next();
        	p2.myTiePoint = new Point(x, y);
        	
        	y = y + inc2;
        }
        
        createTipps(xStart, x, yStart, inc1, inc2, STEP);

        Interpolator.run = false;
		myFinalGame.init();
		Interpolator.reset();
		Interpolator.run = true;
	}


}
