package tournament;

import java.awt.BorderLayout;
import java.awt.Dialog;
import java.awt.EventQueue;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import javax.swing.*;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableModel;

public class TournamentEditor extends JFrame {


	String fields[] = { "Left Tree", "Right Tree"}; 
	
	public int rowCount = 0;
	private ArrayList<String> leftTree = new ArrayList<String>();
	private ArrayList<String> rightTree = new ArrayList<String>();
	
	private HashMap<String, String> changedValues = new HashMap<String, String>();

	private Tournament tournament;
	
	public boolean newTableCreated = false;

	public TournamentEditor(Tournament tournament)
	{
		super("Tournament Editor");
		this.tournament = tournament;
		setData(Player.getAllNames());
		setSize(300, 400);
		setAlwaysOnTop(true);
		this.setLocation(tournament.getLocationOnScreen().x + tournament.getSize().width/2 - this.getSize().width/2,
				tournament.getLocationOnScreen().y + tournament.getSize().height/2 - this.getSize().height/2);
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent we)
			{
				dispose();
			}
		});
		init();
		setVisible(true);
	}
	
	public void setData(List<String> left, List<String> right)
	{
		leftTree.clear();
		rightTree.clear();
		for (String l:left)
			leftTree.add(l);
		for (String r:right)
			rightTree.add(r);
	}
	
	public void createEmptySet(int dimension)
	{
		ArrayList<String> left = new ArrayList<String>();
		ArrayList<String> right = new ArrayList<String>();
		for (int i = 0; i < dimension/2; i++)
		{
			left.add("Team"+(i+1));
			right.add("Team"+(i+(dimension/2)+1));
		}
		setData (left, right);
	}
	
	public void setData(List<String> teams)
	{
		setData(teams.subList(0, teams.size()/2), teams.subList(teams.size()/2, teams.size()));
	}

	public void init()
	{
		changedValues.clear();
		newTableCreated = false;
	    TableModel dataModel = new AbstractTableModel() {
			public String getColumnName(int column)
			{
				if (column == 0)
					return "Left Tree";
				else
					return "Right Tree";
			}

			public int getColumnCount()
			{
				return 2;
			}

			public int getRowCount()
			{
				return leftTree.size();
			}

			public Object getValueAt(int row, int col)
			{
				if (col == 0)
					return leftTree.get(row);
				if (col == 1)
					return rightTree.get(row);
				return "";
			}

			@Override
			public boolean isCellEditable(int rowIndex, int columnIndex)
			{
				return true;
			}

			@Override
			public void setValueAt(Object value, int rowIndex, int col)
			{
				System.out.println(value);
				if (col == 0)
				{
					changedValues.put(leftTree.get(rowIndex), value.toString());
					leftTree.set(rowIndex, value.toString());
				}
				if (col == 1)
				{
					changedValues
							.put(rightTree.get(rowIndex), value.toString());
					rightTree.set(rowIndex, value.toString());
				}
			}
		};
		final JTable jt = new JTable(dataModel);

		jt.setRowSelectionAllowed(false);
		jt.setCellSelectionEnabled(true);
		jt.setSurrendersFocusOnKeystroke(true);
		JScrollPane pane = new JScrollPane(jt);
		getContentPane().add(pane, BorderLayout.CENTER);

		JPanel btnpanel = new JPanel();
		JPanel jpanel = new JPanel();
		jpanel.setLayout(new FlowLayout());
		
		final JComboBox combobox = new JComboBox(new String[] {"2", "4", "8", "16", "32", "64", "128"});
		jpanel.add(combobox);
		JButton btn1 = new JButton("New");
		btn1.addActionListener(new ActionListener(){
			@Override
			public void actionPerformed(ActionEvent e)
			{
				createEmptySet(Integer.parseInt(combobox.getSelectedItem().toString()));
				jt.revalidate();
				newTableCreated = true;
			}
		});
		jpanel.add(btn1);
		JButton btn2 = new JButton("Apply");
		btn2.addActionListener(new ActionListener(){
			@Override
			public void actionPerformed(ActionEvent e)
			{
				if (!newTableCreated)
				{
					for (String player:changedValues.keySet())
					{
						for (Player p:Player.allPlayersVec)
							if (p.getName().equals(player))
								p.myName = changedValues.get(player);
					}
					tournament.repaint();
				} else 
				{
					System.out.println("EMTPY TREE");
					final ArrayList<String> bla = new ArrayList<String>();
					bla.addAll(leftTree); bla.addAll(rightTree);
					tournament.tournamentData.clear();
					
					tournament.buildTournamentTree(Integer.parseInt(combobox.getSelectedItem().toString()), bla);

					newTableCreated = false;
					
				}

			}
		});
		jpanel.add(btn2);
		JButton btn3 = new JButton("Close");
		btn3.addActionListener(new ActionListener(){
			@Override
			public void actionPerformed(ActionEvent e)
			{
				dispose();
			}
		});
		jpanel.add(btn3);
		
		btnpanel.add(jpanel, BorderLayout.PAGE_END);		
		
		getContentPane().add(btnpanel, BorderLayout.PAGE_END);
		
		
	}
}
