package tournament.xml;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

public class GameResult {
	
	public String result = "";
	private String descr  = "";
	
	private Date date = null;
	
	private boolean valid = false;
	
	public int team1 =  -1;
	public int team2 =  -1;
	private int gameid = -1;
	
	SimpleDateFormat df = new SimpleDateFormat( "yyyy-MM-dd HH:mm" );
	
	/**
	 * Creates a new GameResult with its game ID, but no description and no
	 * result (yet).
	 * @param gameid id of the game
	 */
	public GameResult(int gameid)
	{
		team1 = 0;
		team2 = 0;
		
		this.gameid = gameid;
		
		Calendar calendar = Calendar.getInstance();
		calendar.setTime( new Date() );
		calendar.add(Calendar.DAY_OF_MONTH, gameid);
				
		date = calendar.getTime();
		
		System.out.println(date);
	}
	
	/**
	 * Creates a new game with a given result, description, game id and the date of the
	 * game. use this constructor to pass strings parsed from xml-files.
	 * @param result the result of the game as string in format: <int>:<int>, eg: "2:1"
	 * @param id the id of the game
	 * @param descr a short description of the game (eg. for its location, time, etc)
	 * @param date date of the game as string
	 */
	public GameResult(String result, String id, String descr, String date)
	{
		if ( (valid = (result.matches("^[0-9]+:[0-9]+$") && id.matches("^[0-9]+$"))) )
		{
			String[] arr = result.split(":");
			team1  = Integer.parseInt(arr[0]);
			team2  = Integer.parseInt(arr[1]);
			gameid = Integer.parseInt(id);
			
			this.result = result;
			this.descr  = descr;
			
			try {
				this.date = df.parse(date);
			} catch (Exception e){
				valid = false;
			}

		}
		
		System.out.println("GameId: " + gameid + " " + team1 + ":" + team2 + " " + date.toString());
	}
	
	/**
	 * Gets the game's id.
	 * @return the game id
	 */
	public int getGameId()
	{
		return gameid;
	}
	
	/**
	 * Checks whether this Game has a winner.
	 * @return true, if the game is not decided yet.<br>false otherwise.
	 */
	public boolean isGameUndecided()
	{
		return team1 == team2;
	}
	
	/**
	 * Checks whether Team1 won this Game,
	 * @return true, if Team1 is the Winner <br>false otherwise.
	 */
	public boolean isTeam1Winner()
	{
		return team1 > team2;
	}
	
	/**
	 * checks if this GameResult contains valid data.
	 * @return true, if this is valid<br>false otherwise
	 */
	public boolean isValid()
	{
		return valid;
	}
	
	/**
	 * Gets the Description of this Game
	 * @return the description
	 */
	public String getDescription ()
	{
		return descr;
	}
	
	/**
	 * Gets the Date of this Game
	 * @return the date
	 */
	public Date getDate()
	{
		return date;
	}
	
	/** 
	 * Gets the Date as a preformatted string, in the format which is used 
	 * for displaying dates and in all hashmaps within this application
	 * @return this game's date as a string
	 */
	public String getDateAsString()
	{
		return df.format(date);
	}
	
	/**
	 * Returns the result of the game. 
	 */
	public String toString()
	{
		return result;
	}
}
