package tournament.xml;

import java.util.Date;
import java.util.List;

public interface ITournamentData {
	
	/**
	 * Gets the number of teams for this tournament.
	 * @return the number of teams.
	 */
	public int getTeamCount();
	
	/**
	 * Gets the Team by its ID.
	 * @param id id of the team
	 * @return the corresponding team object
	 */
	public Team getTeam(int id);
	
	/**
	 * Gets the Team object by its name. 
	 * @param name name of the team
	 * @return the corresponding team object
	 */
	public Team getTeamByName(String name);
	
	/**
	 * Gets a List<Team> of all participating Teams for iteration.
	 * @return List of all teams.
	 */
	public List<Team> getAllTeams();
	
	/**
	 * Gets the number of games in this tournmanet.
	 * @return number of games
	 */
	public int getGameCount();
	
	/**
	 * Gets a game of the tournament by its id.
	 * @param id id of the game
	 * @return the corresponding GameResult
	 */
	public GameResult getGame(int id);
	
	/**
	 * Gets a game of the tournament by its date.
	 * @param date date of the game
	 * @return the corresponding GameResult
	 */
	public GameResult getGameByDate(Date date);
	
	/**
	 * Gets a List<GameResult> of all Games for iteration.
	 * @return a list of Games
	 */
	public List<GameResult> getAllGames();
	
	/**
	 * Gets a list of all Game-DATES which is allready sorted.
	 * @return a list with dates.
	 */
	public List<Date> getAllDatesSorted();
	

}
