package tournament.xml;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

public class TournamentData extends XMLParser implements ITournamentData {
	
	public TournamentData()
	{
		super();
	}
	
	public TournamentData(String fileName)
	{
		super(fileName);
	}
	
	@Override
	public int getTeamCount()
	{
		return teams.size();
	}
	
	@Override
	public Team getTeam(int id)
	{
		if (id > 0 && id < teams.size())
			return teams.get(id);
		
		return null;
	}
	
	@Override
	public List<Team> getAllTeams()
	{
		return teams;
	}
	
	/**
	 * Returns a List<String> of all Teams containing only the names of 
	 * the tames 
	 * @return list of all names
	 */
	public List<String> getAllTeamsAsStrings()
	{
		ArrayList<String> ret = new ArrayList<String>();
		for (Team t :teams)
			ret.add(t.getName());
		
		return ret;
	}

	@Override
	public List<GameResult> getAllGames() 
	{
		return games;
	}

	@Override
	public GameResult getGame(int id) 
	{
		if( resultMap.containsKey(id) == false )
			return new GameResult(id);
		else
			return resultMap.get(id);
	}

	@Override
	public GameResult getGameByDate(Date date) 
	{
		return resultDateMap.get(date);
	}

	@Override
	public int getGameCount() 
	{
		return games.size();
	}

	@Override
	public Team getTeamByName(String name) 
	{
		for (Team team:teams)
		{
			if (team.getName().toLowerCase().equals(name.toLowerCase()))
				return team;
		}
		
		return null;
	}

	@Override
	public List<Date> getAllDatesSorted() 
	{
		return dates;
	}
}
